aboutsummaryrefslogtreecommitdiffstats
path: root/src/config.c
diff options
context:
space:
mode:
Diffstat (limited to 'src/config.c')
-rw-r--r--src/config.c69
1 files changed, 62 insertions, 7 deletions
diff --git a/src/config.c b/src/config.c
index 751b882..de7207d 100644
--- a/src/config.c
+++ b/src/config.c
@@ -27,11 +27,10 @@ static void
usage(void)
{
fprintf(stderr, _("usage: %s %s\n"), argv0,
- _("[-b brightness] [-c config-file] [-D | +D] [-g gamma] "
- "[-l latitude:longitude | -l provider[:options]] "
- "[-m method[:options]] [-P | +P] [-r | +r] [-dv]"
- "[-O temperature | -o | -p | -t temperature | -x] "
- "| -h | -V"));
+ _("[-b brightness] [-c config-file] [-D | +D] [-E | +E | -e elevations] "
+ "[-g gamma] [-l latitude:longitude | -l provider[:options]] "
+ "[-m method[:options]] [-P | +P] [-r | +r] [-dv] "
+ "[-O temperature | -o | -p | -t temperature | -x] | -h | -V"));
exit(1);
}
@@ -435,6 +434,43 @@ set_gamma(char *str, struct setting_f3 *day, struct setting_f3 *night, const cha
/**
+ * Parse and set solar elevation settings
+ *
+ * The fucntion assumes that the setting source is the command line
+ *
+ * @param str The gamma specification to parse
+ * @param day The currently specified lowest solar elevation for
+ * daytime, will be updated; `NULL` if it shall not be set
+ * @param night The currently specified highest solar elevation for
+ * nighttime, will be updated; `NULL` if it shall not be set
+ */
+static void
+set_elevations(char *str, struct setting_f *day, struct setting_f *night)
+{
+ struct setting_f *settings[] = {day, night};
+ const enum setting_source source = SETTING_CMDLINE;
+ char *strs[2], *end;
+ size_t i, j;
+
+ if (!get_strings(str, !!day + !!night, strs, ':')) {
+ invalid:
+ weprintf(_("Malformed solar elevation argument."));
+ eprintf(_("Try `-h' for more information."));
+ }
+
+ errno = 0;
+ for (i = 0, j = 0; i < ELEMSOF(settings); j += settings[i++] ? 1 : 0) {
+ if (!settings[i] || !strs[j] || settings[i]->source > source)
+ continue;
+ settings[i]->source |= source;
+ settings[i]->value = strtod(strs[j], &end);
+ if (errno || *end)
+ goto invalid;
+ }
+}
+
+
+/**
* Parse a time string on either of the formats "HH:MM" and "HH:MM:SS"
*
* Times up to, but excluding, 48:00 are supported.
@@ -607,7 +643,7 @@ load_defaults(struct settings *settings)
memset(settings, 0, sizeof(*settings)); /* set each `.source` to `SETTING_DEFAULT` and booleans to 0 */
settings->config_file = NULL;
- settings->until_death = 0;
+ settings->scheme_type = UNSPECIFIED_SCHEME;
settings->day.temperature.value = DEFAULT_DAY_TEMPERATURE;
settings->day.brightness.value = DEFAULT_DAY_BRIGHTNESS;
@@ -667,6 +703,15 @@ load_from_cmdline(struct settings *settings, int argc, char *argv[])
settings->until_death = 1;
break;
+ case 'E':
+ settings->scheme_type = CLOCK_SCHEME;
+ break;
+
+ case 'e':
+ set_elevations(ARG(), &settings->elevation_high, &settings->elevation_low);
+ settings->scheme_type = SOLAR_SCHEME;
+ break;
+
case 'g':
set_gamma(ARG(), &settings->day.gamma, &settings->night.gamma, NULL);
break;
@@ -789,6 +834,10 @@ load_from_cmdline(struct settings *settings, int argc, char *argv[])
settings->disabled.value = 1;
break;
+ case 'E':
+ settings->scheme_type = SOLAR_SCHEME;
+ break;
+
case 'P':
settings->preserve_gamma.source |= SETTING_CMDLINE;
settings->preserve_gamma.value = 1;
@@ -921,9 +970,15 @@ load_settings(struct settings *settings, int argc, char *argv[])
for (setting = section->settings; setting; setting = setting->next)
load_from_config_ini(settings, setting->name, setting->value);
/* Further validate settings */
n = !!settings->dawn_start.source + !!settings->dawn_end.source;
n += !!settings->dusk_start.source + !!settings->dusk_end.source;
+ if (settings->scheme_type == SOLAR_SCHEME) {
+ n = 0;
+ } else if (settings->scheme_type == CLOCK_SCHEME) {
+ if (!n)
+ eprintf(_("`-E' option used with a time-configuration configured."));
+ }
if (n) {
scheme.type = CLOCK_SCHEME;
if (n != 4)
@@ -921,9 +970,15 @@ load_settings(struct settings *settings, int argc, char *argv[])
for (setting = section->settings; setting; setting = setting->next)
load_from_config_ini(settings, setting->name, setting->value);
- /* Further validate settings */
+ /* Further validate settings and select scheme */
n = !!settings->dawn_start.source + !!settings->dawn_end.source;
n += !!settings->dusk_start.source + !!settings->dusk_end.source;